//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									Корректировка настройки PROGMEM для MEGA2560 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if  defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)	
#include <avr/pgmspace.h>
#ifdef PROGMEM
  #undef PROGMEM
#endif
#define PROGMEM __attribute__(( section(".progmem.data") ))
#endif
///+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//									Стандартные библиотеки 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#if defined(ARDUINO) && (ARDUINO >= 100) 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//								МАКРОСЫ ДЛЯ ВСЕХ ПЛАТФОРМ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_MICROS				micros()						//--Макрос измерения времени в mcs;
#define RT_HW_MILLIS				millis()						//--Макрос измерения времени в ms;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//								МАКРОСЫ ДЛЯ РАБОТЫ C PROGMEM
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_BUILD_BYTE_PGM(a) 	const byte    a[] PROGMEM  		//--Создание в PROGMEM массива byte 
#define RT_HW_BUILD_INT8_PGM(a) 	const int8_t  a[] PROGMEM  		//--Создание в PROGMEM массива int8_t 
#define RT_HW_READ_BYTE_PGM(arr,n) 	pgm_read_byte(&arr[n])			//--Чтение из массива arr байта с номером n(для >64 kB);
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//					МАКРОСЫ ДЛЯ РАБОТЫ C БУФЕРОМ char hive.buffer и строковыми переменными
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_CODE_CHAR_BREAK		'~'			//--Код "Запрещающего" символа;
#define RT_HW_CODE_CHAR_BLANK	    's'			//--Если равен этому символу, "Символ заполнения='пробел', иначе =символу заключенного в кавычках;
//#define RT_HW_CODE_CHAR_NULL	    '\0'		//--Символ указатель, что заполнитель "пробел";
//#define RT_HW_CODE_MODE_UPPER  		0			//--Если =0, предпочителен вывод в нижнем регистре, иначен выбирается верхний регистр;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						РАЗМЕЩЕНИЕ В В PROGMEM НИМЕНОВАНИЙ ТИПОВ ПИНOB 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
enum {RT_HW_PIN_KIND_ALL=0, RT_HW_PIN_KIND_DIN=1, RT_HW_PIN_KIND_ADC=2, RT_HW_PIN_KIND_DOT=3,
      RT_HW_PIN_KIND_PWM=4, RT_HW_PIN_KIND_DAC=5, RT_HW_PIN_KIND_TCH=6, RT_HW_PIN_KIND_DIR=7,
	  RT_HW_PIN_KIND_INT=8, RT_HW_PIN_KIND_QNT=9, RT_HW_PIN_INIT_MODE=15};		 	  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//						ПЕРЕЧИСЛЕНИЯ ИДЕНТИФИКАТОРОВ И ПЛАТФОМ 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
enum {RT_HW_ID_ARCH_AVR=1, RT_HW_ID_ARCH_SAM3X=2, RT_HW_ID_ARCH_ESP8266=3, RT_HW_ID_ARCH_ESP32=4, RT_HW_ID_ARCH_STM32F1=5,  RT_HW_ID_ARCH_ANON=9};   
enum {RT_HW_ID_BOARD_AVR328=10,     RT_HW_ID_BOARD_MEGA2560=15,    RT_HW_ID_BOARD_AVR_ANON=19,
      RT_HW_ID_BOARD_SAM3X8E=20,    RT_HW_ID_BOARD_SAM3X_ANON=29,
	  RT_HW_ID_BOARD_ESP8266=30,    RT_HW_ID_BOARD_ESP8266_ANON=39,
	  RT_HW_ID_BOARD_ESP32=40,      RT_HW_ID_BOARD_ESP32_DEV=41,   RT_HW_ID_BOARD_ESP32_ANON=49,	 
	  RT_HW_ID_BOARD_STM32F103C=50, RT_HW_ID_BOARD_STM32F103R=51,  RT_HW_ID_BOARD_STM32F103V=52, RT_HW_ID_BOARD_STM32F103Z=53, RT_HW_ID_BOARD_STM32F1_ANON=59};   	  	
//*************************************************************************************************
//								I. ПЛАТФОРМА AVR
//*************************************************************************************************
#if defined(__AVR__)
#define	RT_HW_ARCH_AVR												//--Идентификатор платформы AVR;
#define RT_HW_DATE_TYPE 		uint8_t								//--Базовый тип данных;
#define RT_HW_ARCH_CODE 		RT_HW_ID_ARCH_AVR					//--Код платформы;
#define RT_HW_ARCH_NAME			"AVR"								//--Имя платформы;	
//===============================Контроллеры типа Arduino UNO,Nano=================================
#if   defined(__AVR_ATmega328__) || defined (__AVR_ATmega328P__) || defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) 
#define RT_HW_BOARD_AVR328											//--Идентификатор контроллера
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_AVR328				//--Код контроллера;
#define RT_HW_BOARD_NAME		"AVR328"							//--Наименование контроллера; 
//==============================Контроллеры типа Arduino MEGA2560==================================
#elif   defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__)																
#define RT_HW_BOARD_MEGA2560										//--Идентификатор контроллера
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_MEGA2560				//--Код контроллера;
#define RT_HW_BOARD_NAME		"MEGA2560"							//--Наименование контроллера;
//===============================Другие контроллеры платформы AVR==================================
#else																//--Контроллеры AVR не опознанного типа;
#define RT_HW_BOARD_AVR_ANON										//--Идентификатор контроллера
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_AVR_ANON				//--Код контроллера;
#define RT_HW_BOARD_NAME		"AVR_ANON"							//--Наименование контроллера;
#endif
//-------------------------------------------------------------------------------------------------					
//*************************************************************************************************
//								II. ПЛАТФОРМА ARDUINO DUE
//*************************************************************************************************
#elif defined(__SAM3X8E__) || defined(__SAM3A8C__) || defined(__SAM3A4C__)
#define	RT_HW_ARCH_DUE												//--Идентификатор платформы AVR;
#define RT_HW_DATE_TYPE 		uint32_t							//--Базовый тип данных;
#define RT_HW_ARCH_CODE 		RT_HW_ID_ARCH_SAM3X					//--Код группы;
#define RT_HW_ARCH_NAME			"SAM3x8x"							//--Имя  группы;	
//===============================Контроллеры типа DUE==============================================
#define RT_HW_BOARD_DUE												//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_SAM3X8E				//--Код контроллера;
#define RT_HW_BOARD_NAME		"DUE"							//--Наименование контроллера;
//*************************************************************************************************
//								III. ПЛАТФОРМА  ESP8266
//*************************************************************************************************
#elif defined(ARDUINO_ARCH_ESP8266)
#define	RT_HW_ARCH_ESP8266										//--Идентификатор платформы AVR;
#define RT_HW_ARCH_CODE 		RT_HW_ID_ARCH_ESP8266			//--Код группы;
#define RT_HW_ARCH_NAME			"ESP8266"						//--Имя  группы;	
#define RT_HW_DATE_TYPE 		uint32_t						//--Базовый тип данных;
//===============================Контроллеры типа ESP8266==========================================
#define RT_HW_BOARD_ESP8266										//--Идентификатор контроллера
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_ESP8266			//--Код контроллера;
#define RT_HW_BOARD_NAME		"ESP8266"						//--Наименование контроллера;
//*************************************************************************************************
//								IV. ПЛАТФОРМА  ESP32
//*************************************************************************************************
#elif defined(ARDUINO_ARCH_ESP32)
#define	RT_HW_ARCH_ESP32										//--Идентификатор платформы ESP32;
#define RT_HW_DATE_TYPE 		uint32_t						//--Базовый тип данных;
#define RT_HW_ARCH_CODE 		RT_HW_ID_ARCH_ESP32				//--Код группы;
#define RT_HW_ARCH_NAME		    "ESP32"							//--Имя  группы;							
//===============================Контроллер типа ESP32============================================
#define RT_HW_BOARD_ESP32										//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_ESP32			//--Код контроллера;
#define RT_HW_BOARD_NAME		"ESP32_DEV"						//--Наименование контроллера;
//*************************************************************************************************
//								V. ПЛАТФОРМА  STM32F1
//*************************************************************************************************
#elif defined(ARDUINO_ARCH_STM32)
#define	RT_HW_ARCH_STM32F1										//--Идентификатор платформы STM32F1;
#define RT_HW_DATE_TYPE 		uint32_t						//--Базовый тип данных;
#define RT_HW_ARCH_CODE 		RT_HW_ID_ARCH_STM32F1			//--Код группы;
#define RT_HW_ARCH_NAME			"STM32F1"						//--Имя  группы;	
//===============================Контроллер типа STM32F103C========================================
#if   defined(_BOARD_GENERIC_STM32F103C_H_) 					
#define RT_HW_BOARD_STM32F103C									//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_STM32F103C		//--Код контроллера;
#define RT_HW_BOARD_NAME		"Generic STM32F103C"			//--Наименование контроллера;
//===============================Контроллер типа STM32F103R========================================
#elif defined(_BOARDS_GENERIC_STM32F103R_H_) || defined(_BOARD_GENERIC_STM32F103R_H_) || defined(_BOARD_GENERIC_STM32F103R8_H_) || defined(_BOARD_GENERIC_STM32F103RE_H_) 
#define RT_HW_BOARD_STM32F103R									//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_STM32F103R		//--Код контроллера;
#define RT_HW_BOARD_NAME		"Generic STM32F103R"			//--Наименование контроллера;
//===============================Контроллер типа STM32F103V========================================
#elif defined(_BOARDS_GENERIC_STM32F103V_H_) || defined(_BOARD_GENERIC_STM32F103VB_H_)
#define RT_HW_BOARD_STM32F103V									//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_STM32F103V		//--Код контроллера;
#define RT_HW_BOARD_NAME		"Generic STM32F103V"			//--Наименование контроллера;
//===============================Контроллер типа STM32F103R========================================
#elif defined(_BOARDS_GENERIC_STM32F103Z_H_)
#define RT_HW_BOARD_STM32F103Z									//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_STM32F103Z		//--Код контроллера;
#define RT_HW_BOARD_NAME		"Generic STM32F103Z"			//--Наименование контроллера;
//===============================Контроллер типа STM32F103R========================================
#elif defined(_BOARD_GENERIC_F407V_H_) || defined(_BOARD_DISCOVERY_F4_H_ ) 
#define RT_HW_BOARD_STM32F407V									//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_STM32F407V		//--Код контроллера;
#define RT_HW_BOARD_NAME		"Generic STM32F407V"			//--Наименование контроллера;
//===============================Контроллер типа STM32F103R========================================
#else
#define RT_HW_BOARD_STM32F1_ANON								//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_STM32F1_ANON		//--Код контроллера;
#define RT_HW_BOARD_NAME		"Generic STM32F1_ANON"			//--Наименование контроллера;	
#endif
//*************************************************************************************************
//								VI. ПЛАТФОРМА  ANON (неопознанная платформа)
//*************************************************************************************************
#else
#define	RT_HW_ARCH_ANON											//--Идентификатор контроллера;
#define RT_HW_ARCH_CODE 		RT_HW_ID_ARCH_ANON				//--Код группы;
#define RT_HW_ARCH_NAME			"ANON"							//--Имя  группы;
//===============================Контроллер типа ANON==============================================
#define RT_HW_BOARD_STM32F1_ANON								//--Идентификатор контроллера;
#define RT_HW_BOARD_CODE		RT_HW_ID_BOARD_STM32F1_ANON		//--Код контроллера;
#define RT_HW_BOARD_NAME		"BOARD_STM32F1_ANON"			//--Наименование контроллера;
//-------------------------------------------------------------------------------------------------
#endif			//--Завершение выбора параметров индивидуальных платформ

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//										СТРУКТУРЫ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Структура вывода данных на консоль
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_PRINT_PARAM_INT16_ID   {RT_HW_DATE_TYPE  dir; int16_t  val;};	//--Данные для вывода на консоль int;
struct RT_HW_PRINT_PARAM_UINT16_ID  {RT_HW_DATE_TYPE  dir; uint16_t val;};	//--Данные для вывода на консоль int;
struct RT_HW_PRINT_PARAM_FLOAT_ID   {RT_HW_DATE_TYPE  dir; float    val;};	//--Данные для вывода на консоль int;
struct RT_HW_PRINT_PARAM_UINT16x4_ID{RT_HW_DATE_TYPE  dir,qnt; uint16_t val[4];};//--Данные для вывода на консоль int;
struct RT_HW_PRINT_PARAM_INT16x4_ID {RT_HW_DATE_TYPE  dir,qnt; int16_t val[4];};//--Данные для вывода на консоль int;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		Структуры для работы с пинами
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_PIN_DIN_ID  {RT_HW_DATE_TYPE  dir,cnt; uint16_t device:1, filter:1, normal:1, ago:1, inv:1, step:1, curBlink:1, agoBlink:1;};
struct RT_HW_PIN_ADC_ID  {RT_HW_DATE_TYPE  dir;     uint16_t device; uint16_t filter; uint16_t normal;};
struct RT_HW_PIN_TCH_ID  {RT_HW_DATE_TYPE  dir;     uint16_t device; uint16_t filter; uint16_t normal;};
struct RT_HW_PIN_HALL_ID {RT_HW_DATE_TYPE  dir;     int16_t device;   int16_t filter;  int16_t normal;};
struct RT_HW_PIN_DOT_ID  {RT_HW_DATE_TYPE  dir;};
struct RT_HW_PIN_PWM_ID  {RT_HW_DATE_TYPE  dir,num;uint16_t device; uint16_t freq;};
struct RT_HW_PIN_DAC_ID  {RT_HW_DATE_TYPE  dir;    uint16_t device;};
//-------------------------------------------------------------------------------------------------
struct RT_HW_FIND_I2C_ID {uint8_t dir; uint8_t cnt; uint16_t period; uint32_t timeDelay;};  
//-------------------------------------------------------------------------------------------------											//--Данные для работы с консолью;
struct RT_HW_BASE_VAL_ID {uint32_t val; uint16_t len:4,index:4,num:4,mess:4; uint8_t mode:2,minus:1,point:2,pnt:1;}; //--Структура для выводимой переменной;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++