#include "RT_HW_MAX6675.h"
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.1. Конструктор
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RT_HW_MAX6675::RT_HW_MAX6675() {}
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	1.2 Генератор меандра 50 мс и флага перезапуска устройства (periodReboot в тиках=50мс)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_MAX6675:: genEventID(RT_HW_MAX6675_ID &id){id.event=0; if(RT_HW_MAX6675_MILLIS>=id.timeStart) {id.event=1; id.blink=!id.blink; id.timeStart=RT_HW_MAX6675_MILLIS+RT_HW_MAX6675_PERIOD_TICK;}};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.3. Настройка параметров устройства
// t_sb(период опроса)  =0: 0.5ms; =1: 62.5ms; =2: 125ms; =3: 250ms; =4: 500ms; =5: 1000ms; =6: 10ms; =7: 20ms;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MAX6675:: setID(RT_HW_MAX6675_ID &id, char interface,  uint8_t period, float correct){
if(id.custom==1) {return;} id.custom=1;
//----Установка внешних параметров
id.periodTemper=period; if(id.periodTemper<5) {id.periodTemper=5;}
id.correct=correct;
//----Начальная установка параметров
id.permit=0; id.codeErr=7; id.T=-0.1;	id.err=1; id.fresh=1;	id.first=1;		
//----Определение периодов опроса и допустимой ошибки----------------------------------------------
id.periodErr=100; 												//--Установка периода;
id.cntTemper=id.cntErr=0; 										//--Очистка счетчиков;
if(id.periodTemper) {id.eventTemper=1;}	else {id.eventTemper=0;}//--Установка флага разрешения запуска, если период не равен 0;	
id.enableBlockInterrupts=0;										//--Флаг разрешенения блокировки прерываний при интерфейсе "Программный SPI";	
//----Настройка интерфейса SPI-------------------------------------------------
if(interface=='S') {id.interface='S'; 
					if(id.cs<0) {id.ban=1; return;} id.permit=1;
					digitalWrite(id.cs,1); pinMode(id.cs,OUTPUT); if(!device.spi_ok) {device.spi_ok=1; SPI.begin();}}
//----Настройка интерфейса - программный SPI через 3 пина----------------------
if(interface=='P') {id.interface='P'; 
					if((id.cs<0) || (id.sck<0) || (id.so<0)) {id.ban=1; return;} id.permit=1;
					digitalWrite(id.cs,1); pinMode(id.cs,OUTPUT); pinMode(id.so,INPUT); digitalWrite(id.sck,0);  pinMode(id.sck,OUTPUT);}
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.4. Управление устройством
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MAX6675:: directDevice(RT_HW_MAX6675_ID &id){
genEventID(id); 
//----Установка флагов запуска по времени----------------------------------------------------------
if(id.event){if(id.periodTemper){if(id.periodTemper < 5) {id.periodTemper =5;}
			 if(++id.cntTemper > id.periodTemper) {id.cntTemper=1; if(id.periodTemper) {id.eventTemper =1;}}
			 if(++id.cntErr    > id.periodErr)    {id.cntErr=1; id.err=1; id.T=-0.1; id.fresh=1;}} else {id.eventTemper=0; id.T=-0.1; id.fresh=1;}}
//----Ожидание флага запуска измерения id.eventTemper----------------------------------------------
if(id.stepDirect==0){if(!id.permit) {return;} if(id.eventTemper) {id.eventTemper=0; id.stepDirect++; return; } else {return;}}
//----Чтение сенсора ------------------------------------------------------------------------------
if(id.stepDirect==1){readRawData(id); 
					 if(id.rawData==0)			{id.codeErr=1;  id.stepDirect=0;  return;} //--Выход, ошибочное измерение;
					 if(bitRead(id.rawData,2))	{id.codeErr=2;  id.stepDirect=0; return;} //--Выход, если обрыв термопары;
					 id.adcData=id.rawData>>3; 												//--Контрольное чтение.				 
					 //-------------------------------------------------------------------------------------------
					 if(id.filter){if(id.first) {id.first=0; id.averageData=0;
								   for(uint8_t i=0; i<RT_HW_MAX6675_SIZE_BUFF; i++){id.buff[i]=id.adcFilterData=id.adcData; id.averageData=id.averageData+id.adcData;}}
								   filterAdcData(id);			//--Фильтрация погрешности ADC. 	Выход->id.adcFilterData;
								   filterMiddleData(id);		//--Усреднение измеренных значений. Выход->id.middleFilterData;
								   id.T= (float)id.middleFilterData * RT_HW_MAX6675_RESOLUTION+id.correct; if(id.T<0) {id.T=0;}	
								  /*
								   Serial.print(F("id.filter=")); Serial.print(id.filter);
								   Serial.print(F(" adcData=")); Serial.print(id.adcData);
								   Serial.print(F(" adcFilterData=")); Serial.print(id.adcFilterData);
								   Serial.print(F(" middleFilterData=")); Serial.print(id.middleFilterData);
								   Serial.print(F(" buff[0-3]->"));  
								   Serial.print(id.buff[0]); Serial.print(','); Serial.print(id.buff[1]); Serial.print(',');
								   Serial.print(id.buff[2]); Serial.print(','); Serial.print(id.buff[3]); Serial.print(' ');
								   Serial.print(F("  T="));  Serial.print(id.T,2);  Serial.print('\n');
								  */
								   }
								   
					 else		  {id.T= (float)id.adcData*RT_HW_MAX6675_RESOLUTION+id.correct; if(id.T<0) {id.T=0;}}				   
					 id.fresh=1; id.err=0; id.codeErr=0; id.cntErr=0; 
					 id.stepDirect=0; return;}  
					 };
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.6. Фильтрация погрешности ADC 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MAX6675::filterAdcData(RT_HW_MAX6675_ID &id) {int16_t v; 
v=id.adcFilterData-id.adcData; if((v > 4) || (v < (-4))) {id.adcFilterData=id.adcData;} }		//--Отбрасывание значений с разбросом 4 (0.25)				 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.5. Усреднение измереннызх значений.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MAX6675:: filterMiddleData(RT_HW_MAX6675_ID &id) {int16_t v; 
  if (++id.index>=RT_HW_MAX6675_SIZE_BUFF) id.index = 0; 			//--Перемотка индекса
  id.averageData   -= id.buff[id.index];         			 		//--Вычитание   значения из буфера, на который указывает индекc;
  id.averageData   += id.adcFilterData;          				 	//--Прибавление нового значения;
  id.buff[id.index] = id.adcFilterData;           					//--Запоминание значения  в буфер, на который указывает индекc;
  id.middleFilterData=id.averageData / RT_HW_MAX6675_SIZE_BUFF;		//--Вычислякм новое значение (бегущее среднее арифметическое);
}					 			 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	1.5. Запуск преобразования
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MAX6675::readRawData(RT_HW_MAX6675_ID &id){id.rawData=0;
if(id.interface=='S') {SPI.beginTransaction(SPISettings(4000000, MSBFIRST, SPI_MODE0)); //up to 4MHz, read MSB first, SPI mode 0, see note
					   digitalWrite(id.cs,0);  id.rawData = SPI.transfer16(0x0000);  digitalWrite(id.cs,1); SPI.endTransaction(); } 
if(id.interface=='P') {digitalWrite(id.cs,0);                        
					   if(id.enableBlockInterrupts) {noInterrupts();}         //--Запрет прерываний
					   for (int8_t i=16; i>0; i--){digitalWrite(id.sck,1); id.rawData = (id.rawData<< 1)| digitalRead(id.so); digitalWrite(id.sck,0);}
					   if(id.enableBlockInterrupts) {interrupts();}                                
					   digitalWrite(id.cs,1);}	}
//=================================================================================================	
RT_HW_MAX6675 RT_HW_max6675; //--Создание объекта 

