//***************************************************************************************************
//  Тест библиотеки сенсора MAX6675 (температура 0-1024град.С)
//  Библиотека ориентирована для работы в системах реальногов времени.
//  Библиотека адаптирована для работы с системой визуального програмиирования FLProg.
//  1. Поддерживает подключение через аппаратный SPI и программный SPI (пина).
//  2. Поддерживает одновременную работу с несколькими сенсорами (в т.ч. и с разными интерфейсами подключения);
//  3. Точность измерения  0.25 градусов С.
//  4. Измерение производится через программные прерывания - длительность цикла не более .
//  5. Периодичность вызова определяется параметрами id.periodTemper(default=20) в тиках. Максимальное значение 255. 1 тик=50мс. 
//  6. При обрыве интерфейса или термопары на время более 5сек (параметр id.periodErr=100) взводится флаг id.err и устанавливаются значения по умолчанию
//      id.T=NAN);  
//  7. При обновлении данных взводится флаг id.fresh, который сбрасывается пользователем.
//  8. Доступно два выходных значения в формате float - мгновенное при id.filter=0, 
//      и бегущее среднее арифметическое по четырем значениям        при id.filter=1;
//  9. Период опроса задается в тиках. Один тик =50мс. Если период опроса  меньше 5, ему присваивается значение 5(соответствует 250мс).  
// 10. Длительность цикла опроса с обработкой при id.filter=0 для AV328:  аппаратный SPI=75 мкс; программный SPI=410мкс;
// 11. Контрольные пины тест соответствуют MEGA2560.
//     Для тестирования с другими контроллерами требуется изменить или закомментировать строки 22-23,30-32,43-44.
//***************************************************************************************************
#include "RT_HW_MAX6675.h"
RT_HW_MAX6675_ID id,ids;        //--Создание объекта с данными под сенсор.
uint8_t pinControl=38;          //--Номер контрольного пина;
uint8_t pinIdS=39;              //--Номер контрольного пина ;
uint8_t pinIdP=40;              //--Номер контрольного пина ;
bool blink;
uint32_t timeStart;
//=================================================================================================
void setup() {Serial.begin(115200);
while(!Serial){;};                         //--Ожидание готовности монитора;
pinMode(pinControl,OUTPUT);                 //--Настройка контрольного пина;
pinMode(pinIdS,OUTPUT);                 //--Настройка контрольного пина;
pinMode(pinIdP,OUTPUT);                 //--Настройка контрольного пина;
Serial.println(F("MEGA2560 ->Start test MAX6675***")); //--Вывод стартового сообщения
}
//=================================================================================================
void loop() {
blink=!blink; digitalWrite(pinControl,blink);  //--Вывод контрольного сигнала 
if((millis()-timeStart)>2000){timeStart=millis(); 
Serial.print(F(" Ts=")); Serial.print(id.T,2); Serial.print(F(" Tp=")); Serial.print(ids.T,2);
Serial.print('\n');
} 
//-------------------------------------------------------------------------------------------------
if(id.stepDirect==1) {digitalWrite(pinIdS,1);} else {digitalWrite(pinIdS,0);} //--Для измерения длительности цикла опроса для аппаратного SPI; 
if(ids.stepDirect==1){digitalWrite(pinIdP,1);} else {digitalWrite(pinIdP,0);} //--Для измерения длительности цикла опроса для программного SPI;  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if(!id.custom) {id.cs=49;  id.sck=-1;  id.so=-1; id.filter=0; RT_HW_max6675.setID(id,'S',10,0.0);}    //--Настройка: 'S' - аппаратный SPI; 10 -период опроса 500мс(50мс*10); 49 - пин cs; 0.0 -корректировка значения;
if(1) {RT_HW_max6675.directDevice(id);}
//if(id.fresh){id.fresh=0; Serial.print(F(" Ts=")); Serial.print(id.T,2); Serial.print(F(" raw=")); Serial.print(id.rawData,HEX);
//                         if(id.codeErr) {Serial.print(F(" err=")); Serial.print(id.codeErr);} else {Serial.print(F(" *** Ok"));}  
//                         Serial.print('\n');   }
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
if(!ids.custom){ids.cs=5; ids.sck=4;  ids.so=6;  ids.filter=0; RT_HW_max6675.setID(ids,'P',20,-1.0);} //--Настройка: 'P' - программный SPI; 30-период опроса 1500мс(50мс*30); 5-пин cs; 4-пин sck; 6-пин so; -1.0 -корректировка значения;
if(1) {RT_HW_max6675.directDevice(ids);}
//if(ids.fresh){ids.fresh=0;  Serial.print(F(" Tp=")); Serial.print(ids.T,2); Serial.print(F(" raw=")); Serial.print(ids.rawData,HEX);
//                           if(ids.codeErr) {Serial.print(F(" err=")); Serial.print(ids.codeErr);} else {Serial.print(F(" *** Ok"));}  
//                           Serial.print('\n');  }
//-------------------------------------------------------------------------------------------------
}
//------------------------------------------------------------------------------------------------
